/* globals.h */

/* VIS-5D version 3.0 */

/*
vis5d program for visualizing five dimensional gridded data sets
Copyright (C) 1990, 1991, 1992  Bill Hibbard, Brian Paul, Dave Santek,
and Andre Battaiola.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#ifndef GLOBALS_H
#define GLOBALS_H


#include <sys/types.h>  /* to get uint type */
#include "matrix.h"
#include "sync.h"


#ifdef ibm
#  define SINGLE_TASK
#endif
#ifdef foo
#  define SINGLE_TASK
#endif


#ifndef NULL
#  define NULL 0
#endif

#define ISOSURF 0
#define HSLICE  1
#define VSLICE  2
#define CHSLICE 3
#define CVSLICE 4
#define HWIND   5
#define VWIND   6
#define TRAJ    7

#define MAXTIMES 400
#define MAXPARMS 30

#define MAXTRAJ 10000
#define TRAJGROUPS 8

#define WINDSLICES 2


/*** 1-byte signed integer: ***/
#ifdef stellar
  typedef char  int_1;
#else
  typedef signed char  int_1;
#endif
/*** 1-byte unsigned integer: ***/
typedef unsigned char  uint_1;
/*** 2-byte signed integer; ***/
typedef short int  int_2;
/*** 2-byte unsigned integer; ***/
typedef unsigned short int  uint_2;



extern int Quiet, Debug;

extern char ProgramName[128];

extern char DataFile[100];


/*** Data Set / Grid stuff ***/
extern int Nr, Nc, Nl;
extern int NumTimes;
extern int NumParms;
extern int TimeStamp[MAXTIMES];
extern int DayStamp[MAXTIMES];
extern char ParmName[MAXPARMS][10];

extern float MinVal[MAXPARMS];   /* Min value for each parameter */
extern float MaxVal[MAXPARMS];   /* Max value for each parameter */

extern int Elapsed[MAXTIMES];    /* time in seconds relative to first step */

/* Geographic location for each time step */
extern float NorthLat[MAXTIMES], SouthLat[MAXTIMES];
extern float WestLon[MAXTIMES], EastLon[MAXTIMES];
extern float TopHgt[MAXTIMES], BottomHgt[MAXTIMES];

extern float LatInc, LonInc, HgtInc; /* Geographic resolution per grid point */

extern float NorthBound, SouthBound; /* max of NorthLat's, min of SouthLat's */
extern float WestBound, EastBound;   /* max of WestLon's, min of EastLon's */
extern float TopBound, BottomBound;  /* max of TopHgt's, min of BottomHgt's */

extern int StaticDomain;  /* True if all NorthLat's, SouthLat's, etc.  equal */

extern int Uparm, Vparm, Wparm;
extern int WindAvailable;   /* U,V,W wind info available? */


extern int NumClones;              /* Number of cloned variables */
extern int CloneTable[MAXPARMS];   /* Clone to real variable translation table */


/* Most recently requested graphics values: */
extern float IsoLevel[MAXPARMS];        /* in [MinVal..MaxVal] */
extern float HSliceInterval[MAXPARMS];
extern float HSliceLevel[MAXPARMS];     /* in [0..Nl-1] */
extern float VSliceInterval[MAXPARMS];
extern float VSliceR1[MAXPARMS];        /* in [0..Nr-1] */
extern float VSliceC1[MAXPARMS];        /* in [0..Nc-1] */
extern float VSliceR2[MAXPARMS];        /* in [0..Nr-1] */
extern float VSliceC2[MAXPARMS];        /* in [0..Nc-1] */
extern float CHSliceLevel[MAXPARMS];    /* in [0..Nl-1] */
extern uint  CHSliceColors[MAXPARMS][256]; /* data value --> color LUT */
extern float CVSliceR1[MAXPARMS];       /* in [0..Nr-1] */
extern float CVSliceC1[MAXPARMS];       /* in [0..Nc-1] */
extern float CVSliceR2[MAXPARMS];       /* in [0..Nr-1] */
extern float CVSliceC2[MAXPARMS];       /* in [0..Nc-1] */
extern uint  CVSliceColors[MAXPARMS][256]; /* data value --> color LUT */
extern float HWindLevel[WINDSLICES];    /* in [0..Nl-1] */
extern float HWindDensity[WINDSLICES];  /* around 1.0 */
extern float HWindScale[WINDSLICES];    /* around 1.0 */
extern float VWindR1[WINDSLICES];       /* in [0..Nr-1] */
extern float VWindC1[WINDSLICES];       /* in [0..Nr-1] */
extern float VWindR2[WINDSLICES];       /* in [0..Nr-1] */
extern float VWindC2[WINDSLICES];       /* in [0..Nr-1] */
extern float VWindDensity[WINDSLICES];  /* around 1.0 */
extern float VWindScale[WINDSLICES];    /* around 1.0 */

extern float TrajStep;
extern float TrajLength;
extern int RibbonFlag;


/*** User Interface ***/
extern int CurTime;  /* the currently displayed time step in [0..NumTimes-1] */
extern int GoTime;   /* animate flag (0=off, -1=backward, 1=forward) */

/* graphics display flags */
extern int DisplaySurf[MAXPARMS];
extern int DisplayHSlice[MAXPARMS];
extern int DisplayVSlice[MAXPARMS];
extern int DisplayCHSlice[MAXPARMS];
extern int DisplayCVSlice[MAXPARMS];
extern int DisplayHWind[WINDSLICES];
extern int DisplayVWind[WINDSLICES];
extern int DisplayTraj[TRAJGROUPS];

extern int ContnumFlag;   /* display contour numbers flag */
extern int TopoFlag;      /* is topography available? */
extern int DisplayTopo;   /* display topography? */
extern int MapFlag;       /* is map available? */
extern int DisplayMap;    /* dispaly map? */
extern int CoordFlag;     /* Geographic (0) or grid(1) coordinate display */
extern int DisplayBox;    /* Display 3-D box */
extern int DisplayClock;  /* Display clock */
extern int PrettyFlag;    /* 'Pretty' rendering mode */
extern int DisplayInfo;   /* Display information flag */

extern int Redraw;        /* set when we want to redraw window */
extern int AnimRate;      /* minimum time between animation steps in msec */

extern int DisplayCursor;



/*** Graphics/ Rendering ***/
#define DEFAULT_SCALE 0.70
extern Matrix  CTM, CTMinv;


/* Size of box in graphics coords */
extern float Xmin, Xmax;  /* West, East */
extern float Ymin, Ymax;  /* South, North */
extern float Zmin, Zmax;  /* Top, Bottom */

extern float Gxscale, Gyscale, Gzscale;


extern float CursorX, CursorY, CursorZ;        /* in graphics coords */
extern float CursorR, CursorC, CursorL;        /* in grid coords */


/* Colors */
extern unsigned long Color[MAXPARMS][5];
extern unsigned long HWindColor[WINDSLICES];
extern unsigned long VWindColor[WINDSLICES];
extern unsigned long TrajColor[TRAJGROUPS];
extern unsigned long *CursorColor;
extern unsigned long BoxColor;
extern unsigned long BgColor;
extern unsigned long LabelColor;



/*** Graphics Data Structures ***/

/* Info about isosurfaces */
struct surface {
   int          lock;
   int          valid;          /* valid/initialized surface flag */
   float        isolevel;       /* the isolevel of the surface */
   int          numverts;       /* number of vertices */
   int_2        *vx, *vy, *vz;  /* 2-byte vertices */
   int_1        *nx, *ny, *nz;  /* 1-byte vertex normals */
   int          numpts;         /* number of iso-vertices (numpts>=numverts) */
   uint_2       *pts;           /* array of indices into v, n arrays */
                                /* 2-byte indices means the largest possible */
                                /* isosurface has 65000 vertices. */
};



/* Info about horizontal contour slices */
struct hslice {
   int    lock;
   int    valid;        /* valid/initialized slice flag */
   float  interval;     /* contour line interval */
   float  level;        /* level of slice in [0..Nl-1] */
   int    num1;         /* number of line segment vertices */
   int_2  *vx1, *vy1;   /* pointer to line segment vertices */
   int    num2;         /* number of 'hidden' line segment vertices */
   int_2  *vx2, *vy2;   /* pointer to 'hidden' line segment vertices */
   int    num3;         /* number of label line segment vertices */
   int_2  *vx3, *vy3;   /* pointer to label line segment vertices */
};


/* Info about vertical contour slices */
struct vslice {
   int    lock;
   int    valid;             /* valid/initialized slice flag */
   float  interval;          /* contour line interval */
   float  r1, c1;            /* 1st corner position in [0,Nr-1],[0,Nc-1] */
   float  r2, c2;            /* 2nd corner position in [0,Nr-1],[0,Nc-1] */
   int    num1;              /* number of line segment vertices */
   int_2  *vx1, *vy1, *vz1;  /* pointer to line segment vertices */
   int    num2;              /* number of 'hidden' line segment vertices */
   int_2  *vx2, *vy2, *vz2;  /* pointer to 'hidden' line segment vertices */
   int    num3;              /* number of label line segment vertices */
   int_2  *vx3, *vy3, *vz3;  /* pointer to label line segment vertices */
};


/* Info about colored horizontal slices */
struct chslice {
   int    lock;
   int     valid;        /* valid/initialized slice flag */
   float   level;        /* level of slice in [0..Nl-1] */
   uint_1  *data;        /* Nr x Nc array of 1-byte color indices */
};


/* Info about colored vertical slices */
struct cvslice {
   int    lock;
   int     valid;             /* valid/initialized slice flag */
   float   r1, c1;            /* 1st corner position in [0,Nr-1],[0,Nc-1] */
   float   r2, c2;            /* 2nd corner position in [0,Nr-1],[0,Nc-1] */
   int     rows, cols;        /* size of slice */
   uint_1  *data;             /* rows x cols array of 1-byte color indices */
};


/* Info about a horizontal wind vector slice */
struct hwind {
   int    lock;
   int    valid;         /* valid/initialized slice flag */
   float  level;         /* level in [0,Nl-1] of slice */
   float  density;       /* relative density of vectors (1.0 is nominal) */
   float  scale;         /* vector length scale factor (1.0 is nominal) */
   int    nvectors;      /* number of vectors */
   int_2  *vx, *vy, *vz; /* vertex arrays (4 vertices / vector) */
};


/* Info about a vertical wind vector slice */
struct vwind {
   int    lock;
   int    valid;         /* valid/initialized slice flag */
   float  r1,c1,r2,c2;   /* position in [0,Nr-1],[0,Nc-1] */
   float  density;       /* relative density of vectors (1.0 is nominal) */
   float  scale;         /* vector length scale factor (1.0 is nominal) */
   int    nvectors;      /* number of vectors */
   int_2  *vx, *vy, *vz; /* vertex arrays (4 vertices / vector) */
};



struct traj {
   int     lock;
   int     length;        /* total number of vertices */
   int_2   *vx, *vy, *vz; /* array of vertices */
   int_1   *nx, *ny, *nz; /* array of normals if ribbon */
   uint_2  *start;        /* array [NumTimes] first vertex for each timestep */
   uint_2  *len;          /* array [NumTimes] of lengths for each time step */
   int     group;         /* trajectory group */
   int     kind;          /* type of trajectory:  0 = line, 1 = ribbon */
};



extern LOCK Gfx_Lock;  /* to synchronize reads/writes to graphics below */

extern struct surface  SurfTable[MAXPARMS][MAXTIMES];
extern struct hslice   HSliceTable[MAXPARMS][MAXTIMES];
extern struct vslice   VSliceTable[MAXPARMS][MAXTIMES];
extern struct chslice  CHSliceTable[MAXPARMS][MAXTIMES];
extern struct cvslice  CVSliceTable[MAXPARMS][MAXTIMES];
extern struct hwind    HWindTable[WINDSLICES][MAXTIMES];
extern struct vwind    VWindTable[WINDSLICES][MAXTIMES];
extern struct traj     TrajTable[MAXTRAJ];
extern int NumTraj;


/* Text labels */

#define MAX_LABEL 100

struct label {
   char         text[MAX_LABEL];  /* the text of the label */
   int          len;              /* length of label */
   int          x,y;
   int          x1,y1, x2,y2;     /* bounds of drawn text */
   struct label *next;            /* pointer to next label in list */
};


extern struct label *FirstLabel;

#endif
